<script>
    'use strict';

    $('body').on('click', '.sources-staging_folder-select', function (event) {
        const $stagingFolderFileID = $('#id_source-staging_folder_file_id');
        $stagingFolderFileID.val($(this).data('encoded_filename'));
        $stagingFolderFileID.trigger('change');
        event.preventDefault();
    });
</script>
