__title__ = 'Mayan EDMS'
__version__ = '4.4'
__build__ = 0x040301
__build_string__ = 'v4.4-17-g34bf9cc670_Tue Jan 17 04:00:08 2023 -0500'
__django_version__ = '3.2'
__author__ = 'Roberto Rosario'
__author_email__ = 'roberto.rosario@mayan-edms.com'
__description__ = 'Free Open Source Electronic Document Management System'
__license__ = 'Apache 2.0'
__copyright_short__ = '2011 Roberto Rosario'
__copyright__ = '{} {}'.format('Copyright', __copyright_short__)
__website__ = 'https://www.mayan-edms.com'
